var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
import * as testAccountCases from './testDetails/salesforce.cases.accounts.js';
import * as testAuthCases from './testDetails/salesforce.cases.auth.js';
import * as testLocationCases from './testDetails/salesforce.cases.locations.js';
import * as SalesForcer from '../salesforce.js';
import * as SfAPI from '../salesforceAPI.js';
import * as locationObj from '../sfObjects/location.js';
const expect = chai.expect;
chai.use(chaiAsPromised);
describe('Test the SalesForcer Location Level Functions', () => __awaiter(void 0, void 0, void 0, function* () {
    before(() => __awaiter(void 0, void 0, void 0, function* () {
        SfAPI.disconnect();
        fetchMock.restore();
        let authMock = testAuthCases.apiMockCases.goodTokenReq;
        fetchMock.post(authMock.req.url, { status: authMock.req.code, body: authMock.req.body });
        yield SfAPI.connect();
        fetchMock.restore();
    }));
    context('Test direct location search functions', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testLocationCases.locationThatExists.req.url, testLocationCases.locationThatExists.req.body);
            fetchMock.get(testLocationCases.noLocationsFound.req.url, testLocationCases.noLocationsFound.req.body);
        }));
        it('Throws errors for invalid selectors', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            yield expect(SalesForcer.getBD2Locations({ asdf: "lol" })).to.be.rejectedWith('Unsupported filter asdf for query type BD2_Location__c');
        }));
        it('Returns an null value for a location that doesnt exist', () => __awaiter(void 0, void 0, void 0, function* () {
            let dontExist = yield SalesForcer.getBD2Locations({ locationName: "DontExist" });
            expect(dontExist).to.be.empty;
        }));
        it('Gets a location by ID', () => __awaiter(void 0, void 0, void 0, function* () {
            let exists = yield SalesForcer.getBD2Locations({ id: "ARealIdThatExists" });
            expect(exists[0].details).to.deep.equal(testLocationCases.locationThatExists.resp);
        }));
        it('Gets a location by Name', () => __awaiter(void 0, void 0, void 0, function* () {
            let exists = yield SalesForcer.getBD2Locations({ locationName: "ARealPlaceThatExists" });
            expect(exists[0].details).to.deep.equal(testLocationCases.locationThatExists.resp);
        }));
    }));
    context('Test searching for locations by account', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            fetchMock.get(testAccountCases.accountWithoutLocations.req.url, testAccountCases.accountWithoutLocations.req.body);
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
        }));
        it('Returns all locations for an account', () => __awaiter(void 0, void 0, void 0, function* () {
            let account = yield SalesForcer.getBD2Accounts({ accountName: "ACME" });
            let locationList = yield account[0].getLocations({});
            expect(locationList.map((a) => (a.details))).to.deep.equal(testAccountCases.accountByNameOrIDWithLocations.childLocationResp);
        }));
    }));
    context('Test creating New Locations', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            fetchMock.post(/BD2_Location__c\/$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testLocationCases.checkLocationBody(url, opts);
            }));
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
        }));
        it('Validates inputs', () => __awaiter(void 0, void 0, void 0, function* () {
            let newLocation = locationObj.createNew();
            newLocation.details.accountId = "123";
            yield expect(SalesForcer.writeFile(newLocation)).to.be.rejectedWith('Missing mandatory value topLevel -> locationName from input');
            //@ts-ignore
            newLocation.details.accountId = 123;
            newLocation.details.locationName = "Yert";
            yield expect(SalesForcer.writeFile(newLocation)).to.be.rejectedWith('Field accountId was type "number" but must be string');
        }));
        it('Throws exception trying to create for invalid accountId', () => __awaiter(void 0, void 0, void 0, function* () {
            let newLocation = locationObj.createNew();
            newLocation.details.accountId = "accountDoesntExist";
            newLocation.details.locationName = "Yert";
            yield expect(SalesForcer.writeFile(newLocation)).to.be.rejectedWith('Invalid location parent provided');
            newLocation.details.accountId = "malformedId";
            yield expect(SalesForcer.writeFile(newLocation)).to.be.rejectedWith('Invalid location parent provided');
        }));
        it('Throws an exception for duplicate values (SF rejection)', () => __awaiter(void 0, void 0, void 0, function* () {
            let newLocation = locationObj.createNew();
            newLocation.details.accountId = "duplicateId";
            newLocation.details.locationName = "Yert";
            yield expect(SalesForcer.writeFile(newLocation)).to.be.rejectedWith('SF rejected write due to duplicate value on field "Name"');
        }));
        it('Throws an exception for missing mandatory values (SF rejection)', () => __awaiter(void 0, void 0, void 0, function* () {
            let newLocation = locationObj.createNew();
            newLocation.details.accountId = "missingNameField";
            newLocation.details.locationName = "Yert";
            // yeah it's not actually missing bd2_account__c - I cheated in the mock but it doesn't really matter
            yield expect(SalesForcer.writeFile(newLocation)).to.be.rejectedWith('Missing fields: BD2_Account__c | Some_Other_Field__c');
        }));
        it('Creates a location for an account', () => __awaiter(void 0, void 0, void 0, function* () {
            // yeah yeah this isn't really a valid location but I'm lazy
            let account = yield SalesForcer.getBD2Accounts({ accountName: "ACME" });
            let newLocation = account[0].createLocation();
            newLocation.details.locationName = "Yert";
            newLocation.details.address.province = "NY";
            newLocation = (yield SalesForcer.writeFile(newLocation));
            expect(newLocation.details.id).to.equal('newLocationId');
        }));
    }));
    context('Test updating Locations', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.patch(/BD2_Location__c\/\w+?$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testLocationCases.checkLocationBody(url, opts);
            }));
        }));
        it('Updates location fields', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingLocation = locationObj.createNew();
            existingLocation.loadedFromSF = true;
            existingLocation.details.id = "thingToUpdate";
            existingLocation.details.address.postalCode = "12345";
            existingLocation.details.locationName = "somethingToUpdate";
            existingLocation.details.accountId = "123456";
            yield expect(SalesForcer.writeFile(existingLocation)).not.to.be.rejected;
        }));
        it('Throws an exception if updated fields conflict another entry (rejected by SF)', () => __awaiter(void 0, void 0, void 0, function* () {
            let existingLocation = locationObj.createNew();
            existingLocation.loadedFromSF = true;
            existingLocation.details.id = "conflictingValue";
            existingLocation.details.businessName = "FFFFF";
            existingLocation.details.address.postalCode = "12345";
            existingLocation.details.locationName = "somePlaceWhatever";
            existingLocation.details.accountId = "QQQQQQQQQ";
            yield expect(SalesForcer.writeFile(existingLocation)).to.be.rejectedWith('SF rejected write due to duplicate value on field "Name"');
        }));
    }));
}));
