var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
import * as testAccountCases from './testDetails/salesforce.cases.accounts.js';
import * as testAuthCases from './testDetails/salesforce.cases.auth.js';
import * as testSubCases from './testDetails/salesforce.cases.subs.js';
import * as SalesForcer from '../salesforce.js';
import * as SfAPI from '../salesforceAPI.js';
import * as subObj from '../sfObjects/subscription.js';
const expect = chai.expect;
chai.use(chaiAsPromised);
chai.config.truncateThreshold = 0;
describe('Test the SalesForcer Subscription Level Functions', () => __awaiter(void 0, void 0, void 0, function* () {
    before(() => __awaiter(void 0, void 0, void 0, function* () {
        SfAPI.disconnect();
        fetchMock.restore();
        let authMock = testAuthCases.apiMockCases.goodTokenReq;
        fetchMock.post(authMock.req.url, { status: authMock.req.code, body: authMock.req.body });
        yield SfAPI.connect();
        fetchMock.restore();
    }));
    context('Test direct sub search functions', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testSubCases.noSubsFound.req.url, testSubCases.noSubsFound.req.body);
            fetchMock.get(testSubCases.selectSubById.req.url, testSubCases.selectSubById.req.body);
        }));
        it('Throws errors for invalid selectors', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            yield expect(SalesForcer.getBD2Subs({ asdf: "lol" })).to.be.rejectedWith('Unsupported filter asdf for query type BD2_Subscription__c');
        }));
        it('Returns an null value for a sub that doesnt exist', () => __awaiter(void 0, void 0, void 0, function* () {
            let dontExist = yield SalesForcer.getBD2Subs({ id: "SubDontExist" });
            expect(dontExist).to.be.empty;
        }));
        it('Gets a sub by ID', () => __awaiter(void 0, void 0, void 0, function* () {
            let exists = yield SalesForcer.getBD2Subs({ id: "ARealIdThatExists" });
            expect(exists[0].details).to.deep.equal(testSubCases.selectSubById.resp);
        }));
        it('Gets a sub by Name', () => __awaiter(void 0, void 0, void 0, function* () {
            let exists = yield SalesForcer.getBD2Subs({ externalId: "theNameOfASub" });
            expect(exists[0].details).to.deep.equal(testSubCases.selectSubById.resp);
        }));
    }));
    context('Test searching for subs by account', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
            fetchMock.get(testSubCases.selectSubsBelongingToAccount.req.url, testSubCases.selectSubsBelongingToAccount.req.body);
        }));
        it('Returns all subs for an account', () => __awaiter(void 0, void 0, void 0, function* () {
            let account = yield SalesForcer.getBD2Accounts({ accountName: "ACME" });
            let subList = yield account[0].getSubscriptions();
            expect(subList.map((a) => (a.details))).to.deep.equal(testSubCases.selectSubsBelongingToAccount.resp);
        }));
    }));
    context('Test creating New Subs', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testAccountCases.getAccountNoMatch.req.url, testAccountCases.getAccountNoMatch.req.body);
            fetchMock.post(/BD2_Subscription__c/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testSubCases.checkBodyPost(url, opts);
            }));
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
        }));
        it('Sets default values', () => __awaiter(void 0, void 0, void 0, function* () {
            let newSub = subObj.createNew();
            expect(newSub.details.monies.priceAdjustment).to.equal(0);
            expect(newSub.details.notes).to.equal("");
            expect(Math.abs(Number(newSub.details.monies.invoiceDate) - Number(new Date()))).to.be.lessThan(50);
        }));
        it('Validates inputs', () => __awaiter(void 0, void 0, void 0, function* () {
            let newSub = subObj.createNew();
            newSub.details.accountId = "123";
            yield expect(SalesForcer.createSubscription(newSub)).to.be.rejectedWith('Missing mandatory value topLevel -> qty from input');
            newSub.details.qty = 123;
            //@ts-ignore
            newSub.details.schedule = "whenever I wants";
            newSub.details.subType = subObj.objConstants.subTypes.roof;
            newSub.details.coverage.coverageTo = new Date("21-May-2025");
            newSub.details.coverage.coverageFrom = new Date("21-May-1985");
            newSub.details.monies.chargeType = subObj.objConstants.chargeTypes.invoice;
            newSub.details.monies.currency = subObj.objConstants.currencyTypes.cad;
            newSub.details.monies.invoiceDate = new Date("21-May-1985");
            newSub.details.monies.invoiceNumber = "asdasdsad";
            newSub.details.monies.totalCharge = 123;
            newSub.details.monies.subtotal = 123;
            newSub.details.monies.tax = 0;
            yield expect(SalesForcer.writeFile(newSub)).to.be.rejectedWith('Field "schedule" (whenever I wants) is value restricted and must be one of Monthly or Quarterly or Annually');
        }));
        it('Performs extra checks before creation', () => __awaiter(void 0, void 0, void 0, function* () {
            let newSub = subObj.createNew();
            newSub.details.qty = 123;
            newSub.details.accountId = "123";
            newSub.details.schedule = subObj.objConstants.subSchedules.monthly;
            newSub.details.subType = subObj.objConstants.subTypes.roof;
            newSub.details.coverage.coverageFrom = new Date("21-May-2025");
            newSub.details.coverage.coverageTo = new Date("21-May-1985");
            newSub.details.monies.chargeType = subObj.objConstants.chargeTypes.invoice;
            newSub.details.monies.currency = subObj.objConstants.currencyTypes.cad;
            newSub.details.monies.invoiceDate = new Date("21-May-1985");
            newSub.details.monies.invoiceNumber = "asdasdsad";
            newSub.details.monies.totalCharge = 123;
            newSub.details.monies.subtotal = 123;
            newSub.details.shopSubId = "123456";
            newSub.details.monies.tax = 0;
            yield expect(SalesForcer.writeFile(newSub)).to.be.rejectedWith('coverageFrom must be before coverageTo');
        }));
        it('Throws exception trying to create a new subscription for invalid accountId', () => __awaiter(void 0, void 0, void 0, function* () {
            let newSub = subObj.createNew();
            newSub.details.qty = 123;
            newSub.details.accountId = 'badAccountId';
            newSub.details.schedule = subObj.objConstants.subSchedules.monthly;
            newSub.details.subType = subObj.objConstants.subTypes.roof;
            newSub.details.coverage.coverageTo = new Date("21-May-2025");
            newSub.details.coverage.coverageFrom = new Date("21-May-1985");
            newSub.details.monies.chargeType = subObj.objConstants.chargeTypes.invoice;
            newSub.details.monies.currency = subObj.objConstants.currencyTypes.cad;
            newSub.details.monies.invoiceDate = new Date("21-May-1985");
            newSub.details.monies.invoiceNumber = "asdasdsad";
            newSub.details.monies.totalCharge = 123;
            newSub.details.monies.subtotal = 123;
            newSub.details.shopSubId = "123456";
            newSub.details.monies.tax = 0;
            yield expect(SalesForcer.writeFile(newSub)).to.be.rejectedWith('Invalid subscription parent provided');
        }));
        it('Creates a subscription for an account', () => __awaiter(void 0, void 0, void 0, function* () {
            let account = yield SalesForcer.getBD2Accounts({ accountName: "ACME" });
            let newSub = account[0].createSubscription();
            newSub.details.qty = 5;
            newSub.details.schedule = subObj.objConstants.subSchedules.annual;
            newSub.details.subType = subObj.objConstants.subTypes.scanner;
            newSub.details.coverage.coverageTo = new Date("21-May-2025");
            newSub.details.coverage.coverageFrom = new Date("21-May-1985");
            newSub.details.monies.chargeType = subObj.objConstants.chargeTypes.shopify;
            newSub.details.monies.currency = subObj.objConstants.currencyTypes.usd;
            newSub.details.monies.invoiceDate = new Date("21-May-1985");
            newSub.details.monies.invoiceNumber = "asdasdsad";
            newSub.details.monies.totalCharge = 1000000;
            newSub.details.monies.subtotal = 1200000;
            newSub.details.monies.priceAdjustment = -200000;
            newSub.details.notes = "This is a test!";
            newSub.details.shopSubId = "123456";
            newSub.details.monies.tax = 0;
            newSub = (yield SalesForcer.writeFile(newSub));
            expect(newSub.details.id).to.equal('newSubId');
        }));
    }));
    context('Test updating subs', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.patch(/BD2_Subscription__c\/\w+?$/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return testSubCases.checkBodyPatch(url, opts);
            }));
        }));
        let existingSub = subObj.createNew();
        existingSub.loadedFromSF = true;
        existingSub.details.qty = 5;
        existingSub.details.schedule = subObj.objConstants.subSchedules.annual;
        existingSub.details.subType = subObj.objConstants.subTypes.scanner;
        existingSub.details.coverage.coverageTo = new Date("21-May-2025");
        existingSub.details.coverage.coverageFrom = new Date("21-May-1985");
        existingSub.details.monies.chargeType = subObj.objConstants.chargeTypes.shopify;
        existingSub.details.monies.currency = subObj.objConstants.currencyTypes.usd;
        existingSub.details.monies.invoiceDate = new Date("21-May-1985");
        existingSub.details.monies.invoiceNumber = "asdasdsad";
        existingSub.details.monies.totalCharge = 1200000;
        existingSub.details.monies.subtotal = 1200000;
        existingSub.details.monies.priceAdjustment = 0;
        existingSub.details.notes = "This is a test!";
        existingSub.details.accountId = 'someAccountId';
        existingSub.details.shopSubId = "123456";
        existingSub.details.monies.tax = 0;
        it('Throws an exception if ID is missing', () => __awaiter(void 0, void 0, void 0, function* () {
            yield expect(SalesForcer.writeFile(existingSub)).to.be.rejectedWith("Can't update a field without an Id");
        }));
        it('Updates sub fields', () => __awaiter(void 0, void 0, void 0, function* () {
            existingSub.details.id = 'someExistingSub';
            yield expect(SalesForcer.writeFile(existingSub)).not.to.be.rejected;
        }));
    }));
    context(`Testing subscription shop ID retrieval for an account`, () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.get(testAccountCases.accountByNameOrIDWithLocations.req.url, testAccountCases.accountByNameOrIDWithLocations.req.body);
            fetchMock.get(/q=SELECT%20Invoice_Number__c%20FROM%20BD2_Subscription__c%20WHERE%20BD2_Account__c%20=%20%27someRandomIdLookATthis0mg%27/, testSubCases.subsForAcctResp);
        }));
        it(`Works`, () => __awaiter(void 0, void 0, void 0, function* () {
            let account = yield SalesForcer.getBD2Accounts({ accountName: "ACME" });
            let subList = yield (account[0].getAllSubIds());
            expect(subList).to.deep.equal(['5514445258999', '5514408591607', '5514609197303']);
        }));
    }));
}));
